package abstractset;

/**
 * Title:
 * Description:
 * Copyright:    Copyright (c) 2000
 * Company:      Joller-Voss GmbH
 * @author J.M.Joller
 * @version 1.0
 */

import java.util.*;

class FixedSet extends AbstractSet {
    private Object[] array;
    int size = 0;

    FixedSet(int capacity) {
        array = new Object[capacity];
    }

    FixedSet() {
        this(10);  // Default ist 10
    }

    FixedSet(Collection c) {
        array = new Object[(c.size()*110)/100];  // >= 10%
        addAll(c);
    }

    public boolean add(Object e) {
        if (contains(e))
            return false;
        if (size == array.length)
            throw new IllegalStateException();
        array[size++] = e;
        return true;
    }

    public Iterator iterator() {
        // nicht fail-fast Iterator
        return new Iterator() {
            private int cursor = 0;
            private boolean removeProhibited = true;
            public boolean hasNext() {
                return cursor < size;
            }
            public Object next() {
                if (cursor==size)
                    throw new NoSuchElementException();
                removeProhibited = false;
                return array[cursor++];
            }
            public void remove() {
                if (removeProhibited)
                    throw new IllegalStateException();
                array[--cursor] = array[--size];
                array[size] = null;   // Eliminate "false reference"
                removeProhibited = true;
            }
        };
    }

    public int size() {
        return size;
    }
}
