package observer;

/**
 * Title:
 * Description:
 * Copyright:    Copyright (c) 2000
 * Company:      Joller-Voss GmbH
 * @author J.M.Joller
 * @version 1.0
 */

import java.io.*;
import java.util.*;
import java.util.zip.*;

public class Main implements Observer {
    Main(File dir, boolean includeDirectories) {
        FileWalker fw = new FileWalker();

        fw.addObserver(this);
        fw.walk(dir, includeDirectories);
    }

    // Ausgabe des Dateinamens.
    public void update(Observable o, Object arg) {
        File f = (File)arg;
        try {
            System.out.println(f.getCanonicalPath());
        } catch (IOException e) {
            e.printStackTrace();
        }
    }

    public static void main(String[] args) {
        if (args.length < 1 || args.length > 2) {
            System.err.println("Usage: java Main <directory> [-d]");
            System.err.println("-d : include directories");
            System.exit(1);
        }
        boolean includeDirectories = args.length == 2 && args[1].equals("-d");
        new Main(new File(args[0]), includeDirectories);
    }
}

class FileWalker extends Observable {
    // falls includeDirectories  false, sagt der Observer nichts,
    // falls er auf ein Verzeichnis trifft
    void walk(File dir, boolean includeDirectories) {
        if (dir.isDirectory()) {
            if (includeDirectories) {
                setChanged();
                notifyObservers(dir);
            }
            String[] filenames = dir.list();

            // rekursiv durch die Verzeichnisse
            if (filenames != null) {
                for (int i=0; i<filenames.length; i++) {
                    walk(new File(dir, filenames[i]), includeDirectories);
                }
            }
        } else {
            setChanged();
            notifyObservers(dir);
        }
    }
}
