package stringtokenizer;

/**
 * Title:
 * Description:
 * Copyright:    Copyright (c) 2000
 * Company:      Joller-Voss GmbH
 * @author J.M.Joller
 * @version 1.0
 */

import java.util.StringTokenizer;
import java.util.Enumeration;
import java.util.NoSuchElementException;

public class Main {
  public static void main(String[] args) {
  {   // Block 1
      // START main0
      String s = "99 Tassen Java Kaffee";
      StringTokenizer parser = new StringTokenizer(s);
      // COM
      System.out.println("\nInput: " + s);
      System.out.println("Es gibt " + parser.countTokens() +
		   " Eintrge im Pfad");
      // COM
      try {
        while(parser.hasMoreTokens()) {
	  System.out.println(parser.nextToken());
        }
      } catch (NoSuchElementException e) {
        System.out.println(e);
      }
      // END main0
  }
  {
    // Trennzeichen ";"
    // START main1
    String s = "c:\\windows\\command;c:\\dos;c:\\bin;c:\\util";
    StringTokenizer parser = new StringTokenizer(s, ";");
    // COM
    System.out.println("\nInput: " + s);
    System.out.println("Es gibt " + parser.countTokens() +
		   " Eintrge im Pfad");
    // COM
    try {
      while(parser.hasMoreTokens()) {
	System.out.println(parser.nextToken());
      }
    } catch (NoSuchElementException e) {
      System.out.println(e);
    }
    // END main1
  }
  {
    // enum interfaces
    // START main2
    String s = "c:\\windows\\command;c:\\dos;c:\\bin;c:\\util";
    StringTokenizer parser = new StringTokenizer(s, ";");
    try {
      while(parser.hasMoreElements()) {
	System.out.println(parser.nextElement());
      }
    } catch (NoSuchElementException e) {
      System.out.println(e);
    }
    // END main2
  }
  {
    // Beispiele mit mehreren separators
    // START count
    String s = "a = b + c, d = e.";
    StringTokenizer parser = new StringTokenizer(s, ",.", true);
    // COM
    System.out.println("\nInput: " + s);
    // COM
    System.out.println("Es gibt " + parser.countTokens() + " Tokens");
    try {
      while (parser.hasMoreTokens()) {
	System.out.println(parser.nextToken());
      }
    } catch (NoSuchElementException e) {
      e.printStackTrace();
    }
    // END count
  }
  {
    // START quote
    String s = "frontstuff 'Willkommen zu Hause' endstuff";
    StringTokenizer parser = new StringTokenizer(s, " '", true);
    String token, new_delimiter = null;
    boolean look_for_matching_quote = false;
    // COM
    System.out.println("\nInput: " + s);
    // COM
    try {
      while (parser.hasMoreTokens()) {
	if (new_delimiter != null) {
	    token = parser.nextToken(new_delimiter);
	    new_delimiter = null;
	} else {
	    token = parser.nextToken();
	}
	if (token.equals(" "))
	    continue;
	if (token.equals("'")) {
	    if (look_for_matching_quote) {
		new_delimiter = " '";
		look_for_matching_quote = false;
	    } else {
		new_delimiter = "'";  // blanks?
		look_for_matching_quote = true;
	    }
	} else {
	    System.out.println(token);
	}
      }
  } catch (NoSuchElementException e) {
    e.printStackTrace();
  }
  // END quote
  }
  }
}
